#include "e.h"
#include "e_mod_main.h"

typedef struct _Fonts_CFData F_CFData;

struct _Fonts_CFData
{
   /*- BASIC -*/
   char *size;  //Evas_Font_Size size; /**< Units to descript font sizes. Normally a double, but may vary platforms */ 
   char *font;  
 /*- common -*/
   CalFonts *FontsToModify_L;
};
/* local subsystem functions */
static void	      _font_edit_free_data(E_Config_Dialog *cfd, void *data);
static void           _font_edit_fill_data(F_CFData *f_cfdata);
static void          *_font_edit_create_data(E_Config_Dialog *cfd);
static void           _font_edit_free_data(E_Config_Dialog *cfd, void *data);
static int            _font_edit_basic_apply_data(E_Config_Dialog *cfd, void *data);
static Evas_Object   *_font_edit_basic_create_widgets(E_Config_Dialog *cfd,Evas *evas,void *data);
    
#define IFDUP(src, dst) if(src) dst = strdup(src); else dst = NULL

/* externally accessible functions */
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
void 
font_edit_show(void *con,void *FontsToModify)
{
   E_Config_Dialog *cfd;
   E_Config_Dialog_View v;   

   /* methods */
   v.create_cfdata           = _font_edit_create_data;
   v.free_cfdata             = _font_edit_free_data;
   v.basic.apply_cfdata      = _font_edit_basic_apply_data;
   v.basic.create_widgets    = _font_edit_basic_create_widgets;
   v.advanced.apply_cfdata   = NULL;
   v.advanced.create_widgets = NULL;
   /* create config diaolg */
   cfd = e_config_dialog_new(con, _("Font Editor"), NULL, 0, &v, FontsToModify);
}

/* local subsystem functions */
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static void 
_font_edit_fill_data(F_CFData *f_cfdata)
{
   char temp[50];
   /*- BASIC -*/
   IFDUP(f_cfdata->FontsToModify_L->font,f_cfdata->font);
   sprintf(temp,"%d",f_cfdata->FontsToModify_L->size);
   IFDUP(temp,f_cfdata->size);
}
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static void *
_font_edit_create_data(E_Config_Dialog *cfd)
{
   F_CFData *f_cfdata;

   f_cfdata = E_NEW(F_CFData, 1);
   f_cfdata->FontsToModify_L = E_NEW(CalFonts,1);
   if (!f_cfdata) return NULL;

   f_cfdata->FontsToModify_L = cfd->data;
   _font_edit_fill_data(f_cfdata);
   return f_cfdata;
}
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static void
_font_edit_free_data(E_Config_Dialog *cfd, void *data)
{    
   free(data);
}
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static int
_font_edit_basic_apply_data(E_Config_Dialog *cfd, void *data)
{
   F_CFData *f_cfdata;
   int AllGood;

   f_cfdata = data;
  /**basic**/
   IFDUP(f_cfdata->font,f_cfdata->FontsToModify_L->font);
   AllGood = sscanf(f_cfdata->size,"%d",&(f_cfdata->FontsToModify_L->size));
   if (AllGood == 0) {return -1;}

   e_config_save_queue();
   redraw_calendar(f_cfdata->FontsToModify_L->calendar,0);

    return 1;
}

/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static Evas_Object *
_font_edit_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, void *data)
{
   F_CFData *f_cfdata;
   Evas_Object *ol, *o, *o2;
   f_cfdata = data;
   
   ol = e_widget_list_add(evas, 0, 0);
   o = e_widget_table_add(evas, 0);
   o2 = e_ilist_add(evas);

//label on top
   e_widget_table_object_append(o, e_widget_label_add(evas, "Font"),
                                     1, 0, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_label_add(evas, "Size"),
                                     2, 0, 1, 1,
                                     1, 1, 1, 1);

//Font row
   e_widget_table_object_append(o, e_widget_label_add(evas, ""),
                                     0, 1, 1, 1,
                                     1, 1, 1, 1);
   {
        Evas_Object *entry;
        entry = e_widget_entry_add(evas, &f_cfdata->font);
        e_widget_min_size_set(entry, 100, 1);
        e_widget_table_object_append(o, entry,
                                     1, 1, 1, 1,
                                     1, 1, 1, 1);
    } 
    {
    	Evas_Object *entry2;
        entry2 = e_widget_entry_add(evas, &f_cfdata->size);
        e_widget_min_size_set(entry2, 30, 1);
	e_widget_table_object_append(o, entry2,
                                     2, 1, 1, 1,
                                     1, 1, 1, 1);
    }

   e_widget_list_object_append(ol, o, 1, 1, 0.5);

//test 

 
   return ol;
}
