#include "e.h"
#include "e_mod_main.h"

typedef struct _calendar_cfdata CFData;

struct _calendar_cfdata
{
   int temp_ImageYes;
   int temp_UserCS;
   Calendar *calendar;
};

static Evas_Object 	  *_create_widgets(E_Config_Dialog *cfd, Evas *evas, Config *cfdata);
static void 	    	 *_create_data(E_Config_Dialog *cfd);
static void 	    	  _free_data(E_Config_Dialog *cfd, void *data);
static Evas_Object  	 *_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, void *data);
static int 	     	_basic_apply_data(E_Config_Dialog *cfd, void *data);


/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
void
e_int_config_calendar(void *con, void *calendar)
{
   E_Config_Dialog *cfd;
   E_Config_Dialog_View v;
   /* methods */
   v.create_cfdata           = _create_data;
   v.free_cfdata             = _free_data;
   v.basic.apply_cfdata      = _basic_apply_data;
   v.basic.create_widgets    = _basic_create_widgets;
   v.advanced.apply_cfdata = NULL;
   v.advanced.create_widgets = NULL;

   /* create config diaolg for NULL object/data */
   cfd = e_config_dialog_new(con, _("Calendar Settings"), NULL, 0, &v, calendar);
}
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static void *
_create_data(E_Config_Dialog *cfd)
{
   CFData *cfdata;

   cfdata = E_NEW(CFData, 1);
   cfdata->calendar = cfd->data;
   cfdata->temp_ImageYes = cfdata->calendar->conf->ImageYes;
   cfdata->temp_UserCS = cfdata->calendar->conf->UserCS;
   return cfdata;
}
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static void
_free_data(E_Config_Dialog *cfd, void *data)
{
   /* Free the cfdata */
   free(data);
}
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static Evas_Object *
_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, void *data)
{
   /* generate the core widget layout for a basic dialog */
   Evas_Object *o, *of, *ob, *of2, *of3, *fs, *of1,*check;
   E_Radio_Group *rg;
   CFData *cfdata;
   cfdata = data;

   o = e_widget_list_add(evas, 0, 0);

   of1 = e_widget_framelist_add(evas, _("Color/Text Scheme"), 0);
   check = e_widget_check_add(evas, _("User"), &(cfdata->calendar->conf->UserCS));
   e_widget_framelist_object_append(of1,check);
   e_widget_list_object_append(o, of1, 1, 1, 0.5);

   of2 = e_widget_framelist_add(evas, _("Backgroud Image"), 0);
   rg = e_widget_radio_group_new(&(cfdata->calendar->conf->ImageYes));
   ob = e_widget_radio_add(evas, _("With Top Image"), 0, rg);
   e_widget_framelist_object_append(of2, ob);
   ob = e_widget_radio_add(evas, _("Without Top Image"), 1, rg);
   e_widget_framelist_object_append(of2, ob);
   e_widget_list_object_append(o, of2, 1, 1, 0.5);

   of = e_widget_framelist_add(evas, _("First Day of Week"), 0);
   rg = e_widget_radio_group_new(&(cfdata->calendar->conf->DayofWeek_Start));
   ob = e_widget_radio_add(evas, _("Sunday"), 0, rg);
   e_widget_framelist_object_append(of, ob);
   ob = e_widget_radio_add(evas, _("Monday"), 1, rg);
   e_widget_framelist_object_append(of, ob);
   ob = e_widget_radio_add(evas, _("Tuesday"), 2, rg);
   e_widget_framelist_object_append(of, ob);
   ob = e_widget_radio_add(evas, _("Wednesday"), 3, rg);
   e_widget_framelist_object_append(of, ob);
   ob = e_widget_radio_add(evas, _("Thursday"), 4, rg);
   e_widget_framelist_object_append(of, ob);
   ob = e_widget_radio_add(evas, _("Friday"), 5, rg);
   e_widget_framelist_object_append(of, ob);
   ob = e_widget_radio_add(evas, _("Saturday"), 6, rg);
   e_widget_framelist_object_append(of, ob);
   e_widget_list_object_append(o, of, 1, 1, 0.5);
   return o;
}
/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
/**--APPLY--**/
static int
_basic_apply_data(E_Config_Dialog *cfd, void *data)
{
   CFData *cfdata;
   cfdata = data;
   /* Actually take our cfdata settings and apply them in real life */
   e_border_button_bindings_ungrab_all();
   Calendar *calendar;
   calendar = cfd->data;
   int SwitchImage = 0;

   if ((calendar->conf->ImageYes != cfdata->temp_ImageYes) || (calendar->conf->UserCS != cfdata->temp_UserCS))
   {
	SwitchImage = 1;
	calendar->conf->ImageYes = cfdata->calendar->conf->ImageYes;
	if (calendar->conf->ImageYes != cfdata->temp_ImageYes)
	{
		if (cfdata->temp_ImageYes == 0)
	   		cfdata->temp_ImageYes = 1;
		else 
	   		cfdata->temp_ImageYes = 0;
	}
	if (calendar->conf->UserCS != cfdata->temp_UserCS)
	{
		if (cfdata->temp_UserCS)
			cfdata->temp_UserCS = FALSE;
		else
			cfdata->temp_UserCS = TRUE;
	}
   }
 
   e_config_save_queue();
   redraw_calendar(calendar,SwitchImage);
   e_border_button_bindings_grab_all();

   return 1; /* Apply was OK */
}
